/*****************************************************************************************

	Red Rapids 
	797 N. Grove Rd
	Suite 101
	Richardson, TX 75081
	support@redrapids.com


	WaveFront Demo Program

	Desc:	This program shows how to open the Wavefront and configure it.   
	
	Rev History 
			1.1 - Require R value to be set when N value is set. Corrections to BDAC functions.
			1.0	- [12 Nov 2008 ASB (support@redrapids.com)] Inital Release

******************************************************************************************/

#define MAJ_REV	1
#define MIN_REV	1
#define SUB_REV	0x20090115

#include <stdio.h>
#include <stdlib.h>

#include "adapter.h"
#include "wavefront_memmap.h"
#include "wavefront_functions.h"


#ifdef WIN32
	#include <conio.h>
	#include <windows.h>
	#include <malloc.h>
#else
	
#endif

int Verbose;		

int main(int argc, char* argv[]) 
{
	s_Adapter WF0;		/*Handle to Wavefront device*/
	UINT32 Dummy;
	
	//Flags to set from command line
	int ii; 
	UINT32 i_bdac_pos, i_bdac_neg, q_bdac_pos, q_bdac_neg;
	int I_BDACConfig, Q_BDACConfig;
	UINT32 Cmd, Input;
	int setDivSel, setDirectRfp, setRefOutPwr;
	int div_sel, direct_rf, ref_out_pwr;
	int setAttn;
	double attn;
	UINT32 attnInteger;
	int setSynthN;
	UINT32 synthR, synthN;
	int status;

	printf("Red Rapids\n");
	printf("Adapter Wavefront Demo Program v %x.%x.%08x\n", MAJ_REV, MIN_REV, SUB_REV);
	printf("Built %s @ %s\n", __DATE__, __TIME__);
	printf("\n%s\n", Adapter_GetAPIVerInfo(&Dummy));

	
	//Init all flags to default values
	Adapter_Zero(&WF0);
	WF0.DevNum = 0;
	strcpy(WF0.Asy, "M150");

	i_bdac_pos = 0;
	i_bdac_neg = 0;
	q_bdac_pos = 0;
	q_bdac_neg = 0;
	I_BDACConfig = 0;
	Q_BDACConfig = 0;
	setDivSel = 0;
	div_sel = 0;
	setDirectRfp = 0;
	direct_rf = 0;	
	setRefOutPwr = 0;
	ref_out_pwr = 0;
	setAttn = 0;
	attn = 0;
	attnInteger = 0;
	setSynthN = 0;
	synthR = 10;
	synthN = 0;
	status = 0;


	//Parse the cmd line 
	for(ii=1;ii<argc;ii++)
	{
		if(EQUAL(argv[ii],"-help") || EQUAL(argv[ii],"-HELP") || EQUAL(argv[ii],"-?") || EQUAL(argv[ii],"?"))
		{
			printf("Command Usage\n");
			printf("wavefront:\n");
			printf("\t-stat            \tPrint device status \n");
			printf("\t-v <n>           \tEnable verbose output, n=[1:5]\n");
			printf("\t-asy <n>         \tDevice Assembly Number to use\n");
			printf("\t-dev <n>         \tDevice Number to use\n");
			printf("\t-div_sel <n>     \tn=[0|1]\n");
			printf("\t-direct_rf <n>   \tn=[0|1]\n");
			printf("\t-ref_out <n>     \tn=[0|1]\n");
			printf("\t-attn <n>        \tn=(0:31.5 in increments of 0.5dB)\n");
			printf("\t-synthN <n>	   \tn=(250:4000)\n");
			printf("\t\t---Bias DAC options---\n");
			printf("\t-idacpos <n>   \t\tSet I Channel Positive Offset [0x00 - 3FF]\n");
			printf("\t-idacneg <n>   \t\tSet I Channel Negative Offset [0x00 - 3FF]\n");
			printf("\t-qdacpos <n>   \t\tSet Q Channel Positive Offset [0x00 - 3FF]\n");
			printf("\t-qdacneg <n>   \t\tSet Q Channel Negative Offset [0x00 - 3FF]\n");
			exit(-1);
		}
		

		if(EQUAL(argv[ii],"-stat"))
		{
			status = 1;
			if(Verbose>=1) printf("\t>Printing device status...\n");
		}

		if(EQUAL(argv[ii],"-v"))
		{
			Verbose = atoi(argv[ii+1]);
			ii++;
			if(Verbose>=1) printf("\t>Verbose output enabled...\n");
			
		}

		if(EQUAL(argv[ii],"-asy"))
		{
			strcpy(WF0.Asy, argv[ii+1]);
			ii++;
			if(Verbose>=1) printf("\t>Using Asy = %s...\n", WF0.Asy);
		}

		if(EQUAL(argv[ii],"-dev"))
		{
			WF0.DevNum = atoi(argv[ii+1]);
			ii++;
			if(Verbose>=1) printf("\t>Will use device %d...\n", WF0.DevNum);
		}
		
		if(EQUAL(argv[ii],"-div_sel"))
		{
			div_sel = atoi(argv[ii+1]);
			setDivSel = 1;
			ii++;
			if(Verbose>=1) printf("\t>div_sel set to %d...\n", div_sel);
		}

		if(EQUAL(argv[ii],"-direct_rf"))
		{
			direct_rf = atoi(argv[ii+1]);
			setDirectRfp = 1;
			ii++;
			if(Verbose>=1) printf("\t>direct_rf set to %d...\n", direct_rf);
		}

		if(EQUAL(argv[ii],"-ref_out"))
		{
			ref_out_pwr = atoi(argv[ii+1]);
			setRefOutPwr = 1;
			ii++;
			if(Verbose>=1) printf("\t>ref_out set to %d...\n", ref_out_pwr);
		}

		if(EQUAL(argv[ii],"-attn"))
		{
			attn = atof(argv[ii+1]);
			if(Verbose>=1) printf("\t>Setting attenuation to %d...\n", attn);
			if(attn > 31.5)
			{
				printf("Only attenuation values 0:31.5 are allowed!\n");
				return -1;
			}
			attnInteger = (UINT32)(0 - (2*attn) + 63); //Calculate value to send to attenuator programming interface
			setAttn = 1;
			ii++;
		}

		if(EQUAL(argv[ii],"-synthN"))
		{
			synthN = atoi(argv[ii+1]);
			setSynthN = 1;
			ii++;
			if(Verbose>=1) printf("\t>Setting synthesizer N value to %d...\n", synthN);
		}
		
		if(EQUAL(argv[ii],"-idacpos"))
		{
			i_bdac_pos = atoi(argv[ii+1]);
			I_BDACConfig = 1;
			if(Verbose>=1) printf("\tSetting I-Channel positive DC offset to %d...\n", i_bdac_pos);
			ii++;
		}

		if(EQUAL(argv[ii],"-idacneg"))
		{
			i_bdac_neg = atoi(argv[ii+1]);
			I_BDACConfig = 1;
			if(Verbose>=1) printf("\tSetting I-Channel negative DC offset to %d...\n", i_bdac_neg);
			ii++;
		}

		if(EQUAL(argv[ii],"-qdacpos"))
		{
			q_bdac_pos = atoi(argv[ii+1]);
			Q_BDACConfig = 1;
			if(Verbose>=1) printf("\tSetting Q-Channel positive DC offset to %d...\n", q_bdac_pos);
			ii++;
		}

		if(EQUAL(argv[ii],"-qdacneg"))
		{
			q_bdac_neg = atoi(argv[ii+1]);
			Q_BDACConfig = 1;
			if(Verbose>=1) printf("\tSetting Q-Channel negative DC offset to %d...\n", q_bdac_neg);
			ii++;
		}
			
	}	

	//Open hardware link
	if(Adapter_Open(&WF0, BRG_REV_ADR))
	{
		fprintf(stderr, "Failed to find and/or open card. (%x)\n", WF0.ReturnStatus);
		return -1;
	}
	else
	{
		if(Verbose>=3) printf("\tOpened Adapter device %d at [%d:%x.%x]\n", WF0.DevNum, WF0.Bus, WF0.Slot, WF0.Function);
	}

	if(status)
	{
		Adapter_Read32(&WF0, BAR2, WF_FWREV_ADR, &Dummy); 
		printf("\t>Wavefront Firmware Revision Number: %x\n", (Dummy&0x7));
		Adapter_Read32(&WF0, BAR2, BOARD_CTL_ADR, &Dummy); 
		if(Dummy & INT_REF_ACTIVE)
			printf("\t>Internal reference clock active\n");
		else
			printf("\t>Internal reference clock inactive\n");
		if(Dummy & EXT_LO_ACTIVE)
			printf("\t>External LO active\n");
		else
			printf("\t>External LO inactive\n");
		if(Dummy & INT_LO_ACTIVE)
			printf("\t>Internal LO active\n");
		else
			printf("\t>Internal LO inactive\n");
		printf("\n");
	}

	if(setDivSel)
	{
		/*Set clock source using div_sel:

		div_sel | Ext Samp Clk		  | CLK source 
				| connected to board  |
		---------------------------------------------------------
		   0    |      NO             | Synth
		   0    |      YES            | Ext Sample Clk
		   1    |      NO             | Synth / 2
		   1    |      YES            | N/A, no clk source
		*/
		Adapter_Read32(&WF0, BAR2, BOARD_CTL_ADR, &Dummy); //Read current board control settings
		Dummy = Dummy & 0xfe; //zero out the div_sel bit
		Dummy |= div_sel; //OR in the desired div_sel bit value
		Adapter_Write32(&WF0, BAR2, BOARD_CTL_ADR, Dummy); 
	}
	
	//Set synthesizer values if desired
	//If setting N value, must write to R value as well
	if(setSynthN)
	{
		Adapter_uSleep(1500); //sleep for 1.5ms before allowing N value to be loaded
		SYNTH_WBSx000_SDI_RdWrN(&WF0, 1, synthN);
		Adapter_uSleep(1500); //sleep for 1.5ms before allowing R value to be loaded
		SYNTH_WBSx000_SDI_RdWrN(&WF0, 0, synthR);
	}

	if(setDirectRfp)
	{
		/*Set I channel bypass and LO RF using direct_rf:
		direct_rf  | Result
		--------------------
		   0       | I channel bypass, and use RF OUTPUT
		   1       | I channel on, and use LO RF
		*/
		Adapter_Read32(&WF0, BAR2, BOARD_CTL_ADR, &Dummy);
		Dummy = Dummy & (0xfb); //zero out the direct_rf bit
		Dummy |= (direct_rf<<2); //OR in the desired direct_rf bit value
		Adapter_Write32(&WF0, BAR2, BOARD_CTL_ADR, Dummy); 
	}
	
	if(setRefOutPwr)
	{
		/*Enable or disable REF OUT */
		Adapter_Read32(&WF0, BAR2, BOARD_CTL_ADR, &Dummy);
		Dummy = Dummy & (0xfd); //zero out the ref_out_pwr bit
		Dummy |= (ref_out_pwr<<1); //OR in the desired ref_out_pwr bit value
		Adapter_Write32(&WF0, BAR2, BOARD_CTL_ADR, Dummy); 
	}

	//If desired, set the attenuation
	if(setAttn)
	{
		ATTN_HMC624LP4_SDI_RdWrN(&WF0, attnInteger);
	}

	//If desired perform I/Q Bias DAC configuration
	/* Cmd options:
	0x1 - load positive bias value
	0x2 - load negative bias value
	0x9 - load and update positive bias DAC output
	0xa - load and update negative bias DAC output
	0xe - Bias DAC sleep
	*/

	//DACSel = 0 for Q-channel Bias DAC or 1 for I-channel Bias DAC
	if(I_BDACConfig)
	{
		//Load and update the positive bias output
		Cmd = 0x9;
		Input = i_bdac_pos;
		BDAC2_LTC1661_SDI_RdWrN(&WF0, 0x1, Cmd, Input);
		//Load and update the negative bias output
		Cmd = 0xa;
		Input = i_bdac_neg;
		BDAC2_LTC1661_SDI_RdWrN(&WF0, 0x1, Cmd, Input);
	}
	if(Q_BDACConfig)
	{
		//Load and update the positive bias output
		Cmd = 0x9;
		Input = q_bdac_pos;
		BDAC2_LTC1661_SDI_RdWrN(&WF0, 0x0, Cmd, Input);
		//Load and update the negative bias output
		Cmd = 0xa;
		Input = q_bdac_neg;
		BDAC2_LTC1661_SDI_RdWrN(&WF0, 0x0, Cmd, Input);
	}
	
	//Close the card and exit
	Adapter_Close(&WF0);
	return 0;
}


void Adapter_ISR(s_Adapter *pCA)
{
	//Does not use interrupts
	return;
}



	
