#include <memory.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef WIN32
	#define STR_EQUAL(str1, str2)(_stricmp(str1,str2) == 0) //For cmd line parsing
#else
	#define STR_EQUAL(str1, str2)(strcasecmp(str1,str2) == 0) //For cmd line parsing
#endif

#include "adapter_returns.h"
#include "adapter.h"
#include "wavefront_memmap.h"


/*
	int EQUAL(char * str1, char * str2)
	
	Does case insensitive comparions of two strings.  Returns 1 if strings are the same
*/
int EQUAL(
		  char * str1, 
		  char * str2
		  )
{
	return STR_EQUAL(str1, str2);
}

/*	UINT32 BDAC2_LTC1661_SDI_RdWrN(s_Adapter *pCA, int RdWrN, short Adr, UINT32 Data)
	Performs configuration of the LTC1661 bias DACs. Command word is 16 bits (MSB to LSB):
	cmd, cmd, cmd, cmd, input, input, input, input, input, input, input, input, input, input, X, X */
UINT32 BDAC2_LTC1661_SDI_RdWrN(s_Adapter *pCA, int DACSel, short Ctrl, UINT32 Input)
{
	UINT32 CmdWrd = 0;    //Ctrl command / Input pair to be written
	UINT32 DAC_CTL_Value=0;
	UINT32 SDIProgReg = 0;
	UINT32 BDAC_CSB_SELECT = 0;
	UINT32 BDAC_CSB_DISABLE = 0;
	int x;
	
	/* DACSel= 1-BDAC-I, 0-BDAC-Q */
	if(DACSel == 0)
	{
		BDAC_CSB_SELECT = 0x1<<4; //bit 4, CSB for BDAC-Q
		BDAC_CSB_DISABLE = 0x1<<5; 
	}
	else if(DACSel == 1)
	{
		BDAC_CSB_SELECT = 0x1<<5;//bit 5, CSB for BDAC-I
		BDAC_CSB_DISABLE = 0x1<<4;
	}

	//Form the command word
	CmdWrd = ((Ctrl&0xF)<<12)| ((Input&0x3FF)<<2) | (0x3);  //Lower 2 bits are dont-care
	//printf("Command word is 0x%x\n", CmdWrd);
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
	SDIProgReg = SDIProgReg | 0x38 ; //force PLL_ENABLE(high) and IDAC/QDAC CSB bits(high) inactive
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SDIProgReg);
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &DAC_CTL_Value);
	DAC_CTL_Value = DAC_CTL_Value & ~0x4; //SCLK must go low before CS goes low to start the programming cycle
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, DAC_CTL_Value);
	DAC_CTL_Value = DAC_CTL_Value & ~BDAC_CSB_SELECT;  //BDAC CSB must go low to start programming cycle
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, DAC_CTL_Value); 

	for(x=0; x<=15; x++) 	//Loop through 16 bits of Cmd / Input
	{
		//printf("\tCommand bit is 0x%x\n", (CmdWrd>>15)&0x1);
		Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, 0x8 | BDAC_CSB_DISABLE | (((CmdWrd>>15)&0x1)<<1)); //Shift into the SDI bit for writing to the DAC
		Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
		Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, 0x8 | BDAC_CSB_DISABLE | 0x4 | (((CmdWrd>>15)&0x1)<<1)); //set SLCK to transfer the bit
		Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
		CmdWrd = (CmdWrd<<1);            //shift the MSB off
	}
	
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
	SDIProgReg = SDIProgReg | 0x8 | BDAC_CSB_DISABLE ; //force all other CSB bits inactive
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SDIProgReg); 
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &DAC_CTL_Value);
	DAC_CTL_Value = DAC_CTL_Value | BDAC_CSB_SELECT; // BDAC CSB must go high to end programming cycle
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, DAC_CTL_Value); 
	
	pCA->ReturnStatus = 0x0;
	return pCA->ReturnStatus;
}


UINT32 ATTN_HMC624LP4_SDI_RdWrN(s_Adapter *pCA, UINT32 Ctrl)
{
	UINT32 CmdWrd = 0;    //6-bit control word
	UINT32 ATTN_CTL_Value=0;
	UINT32 SDIProgReg = 0;
	int x;

	//Form the command word
	CmdWrd = Ctrl;
//	printf("Command word is 0x%x\n", CmdWrd);
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
	SDIProgReg = SDIProgReg | 0x38 ;//force CSB bits inactive (IDAC and QDAC go high) and PLL must remain high
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SDIProgReg); 
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &ATTN_CTL_Value);
	ATTN_CTL_Value = ATTN_CTL_Value & ~0x44; //SCLK and ATT_LE must go low to start the programming cycle
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, ATTN_CTL_Value);

	for(x=0; x<=5; x++) 	//Loop through 6 bits of Cmd / Input
	{
		//printf("\tCommand bit is 0x%x\n", (CmdWrd>>5)&0x1);
		Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, 0x38 | (((CmdWrd>>5)&0x1)<<1)); //Shift into the SDI bit for writing to the DAC (Keep all CSBs inactive, unset SCLK)
		Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
		SDIProgReg = SDIProgReg | 0x4;
		Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SDIProgReg); //set SLCK to transfer the bit (Keep all CSBs inactive)
		Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
		CmdWrd = (CmdWrd<<1);            //shift the MSB off
	}

	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
	SDIProgReg = (SDIProgReg | 0x38) & ~(0x2) ;//force CSB bits inactive (IDAC and QDAC go high) and SDI low, and PLL must remain high
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SDIProgReg); 
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &ATTN_CTL_Value);
	ATTN_CTL_Value = ATTN_CTL_Value & ~(SPI_SCLK_HI); // SCLk must go low
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, ATTN_CTL_Value); 
	ATTN_CTL_Value = ATTN_CTL_Value | ATT_LE_HI; // ATT_LE must go high to end programming cycle
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, ATTN_CTL_Value); 
	ATTN_CTL_Value = ATTN_CTL_Value & ~(ATT_LE_HI); // ATT_LE set back to low
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, ATTN_CTL_Value); 
	
	pCA->ReturnStatus = 0x0;
	return pCA->ReturnStatus;
}

UINT32 SYNTH_WBSx000_SDI_RdWrN(s_Adapter *pCA, UINT32 Ctrl, UINT32 Input)
{
	/* Ctrl: 0-update 15-bit R value
			 1-update 23-bit N value */
	UINT32 CmdWrd = 0;    //24-bit control word
	UINT32 SYNTH_CTL_Value=0;
	UINT32 SDIProgReg = 0;
	int x;

	//Form the command word
	CmdWrd = (((Input&0x7FFFFF)<<1) | (Ctrl&0xF));  //Bits 23:1 are R/N value, bit 0 is control
	//printf("Command word is 0x%x\n", CmdWrd);
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
	SDIProgReg = SDIProgReg | 0x30 ; //force PLL_ENABLE(low) and CSB bits(high) inactive (IDAC and QDAC)
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SDIProgReg); 
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SYNTH_CTL_Value);
	SYNTH_CTL_Value = SYNTH_CTL_Value & ~(0xc); //SCLK and PLL_ENABLE must go low to start the programming cycle
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SYNTH_CTL_Value);

	for(x=0; x<=23; x++) 	//Loop through 24 bits of Cmd / Input
	{
		//printf("\tCommand bit is 0x%x\n", (CmdWrd>>23)&0x1);
		Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, (0x30 |(((CmdWrd>>23)&0x1)<<1))); //Shift into the SDI bit for writing to the Synth (Keep all CSBs inactive, unset SCLK)
		Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
		Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, 0x4 | 0x30 | (((CmdWrd>>23)&0x1)<<1)); //set SLCK to transfer the bit (Keep all CSBs inactive)
		Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
		CmdWrd = (CmdWrd<<1); //shift the MSB off
		Adapter_uSleep(34); //minimum clock frequency
	}
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SDIProgReg);
	SDIProgReg = SDIProgReg | 0x30 ; //force PLL_ENABLE(low) and CSB bits(high) inactive (IDAC and QDAC)
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SDIProgReg); //force all other CSB bits inactive
	Adapter_uSleep(38); //sleep for 38usec before pulsing pll_enable
	Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SYNTH_CTL_Value);
	SYNTH_CTL_Value = (SYNTH_CTL_Value | PLL_ENABLE) & ~(SPI_SCLK_HI); // SCLk must go low and PLL_ENABLE high to update the synthesizer values
	Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SYNTH_CTL_Value); 
	//Adapter_Read32(pCA, BAR2, WF_SERIAL_CTL_ADR, &SYNTH_CTL_Value);
	//SYNTH_CTL_Value = SYNTH_CTL_Value & 0xF3; // SCLk must go low and PLL_ENABLE low to end programming cycle
	//Adapter_Write32(pCA, BAR2, WF_SERIAL_CTL_ADR, SYNTH_CTL_Value); 

	pCA->ReturnStatus = 0x0;
	return pCA->ReturnStatus;
}


